﻿using EvaPosSrvDTO;
using EvaPosSrvResp;
using gatewayGK.Infraestructura;
using GatewaySCO;
using Microsoft.Extensions.Configuration;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace gatewayGK.POSBC
{
    public class InitializeRequestPosbcCmd : IComando
    {
        public string Referencia { get; set; } = "scsns:Initialize";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public InitializeRequestDTO Request { get; private set; }
        /// <summary>
        /// Nos traemos la trama de entrada
        /// </summary>
        public TramaSCO Trama {  get; private set; }

        /// <summary>
        /// Procesa y responde InitializeRequest.
        /// </summary>
        public Respuestas Ejecutar()
        {
            IConfiguration configBuilder = ConfigurationHelper.Configuration;


            string direccionIpPosbc = configBuilder["posbc:DireccionIpPosbc"];
            string puertoPosbcString = configBuilder["posbc:PuertoPosbc"];
            int puertoPosbcInt = int.Parse(puertoPosbcString);
            string xmlData = Request.mensajeXml;

            
            Log.Information($"Inicio del comando InitializeRequest para realizar conexión con el POSBC");
            Log.Information($"Dirección IP del servidor POSBC {direccionIpPosbc}, Puerto de conexión: {puertoPosbcInt}");
            // Convertir la trama XML a bytes
            byte[] dataToSend = Encoding.UTF8.GetBytes(xmlData);
            byte[] bufferRecibido = Entorno<EntornoPOSBC>.Instancia.get().BufferEntrada;
            int cantBytes = Entorno<EntornoPOSBC>.Instancia.get().CantBytes;
            Log.Information($"Tamaño del buffer recibido desde CHEC: {bufferRecibido}");
            Log.Information($"Cantidad de bytes recibido desde CHEC: {cantBytes}");
            try
            {
                // Crear un socket TCP/IP
                Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                // Configurar el endpoint (IP y puerto)
                IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(direccionIpPosbc), puertoPosbcInt);

                //Conectar al endpoint
                socket.Connect(endPoint);
                Log.Information("Conectandose al servidor POSBC...");


                // Enviar los datos
                socket.Send(bufferRecibido, SocketFlags.None);
                Log.Information($"Buffer de entrada enviado al servidor POSBC {bufferRecibido}");

                // Configurar timeout para el socket (por ejemplo, 5 segundos)
                socket.ReceiveTimeout = 5000;

                // Buffer para recibir la respuesta - Revisar cual es la mejor manera de recibir ese
                List<byte> receivedBytes = new List<byte>();
                byte[] bufferSalida = new byte[1024];

                while (true)
                {
                    int bytesLeidos = 0;
                    if (socket.Poll(5000000, SelectMode.SelectRead))
                    {
                        bytesLeidos = socket.Receive(bufferSalida, 0, bufferSalida.Length, SocketFlags.None);

                        //Coloque 1 Porque POSBC si no tiene mas bytes que mandar, manda 1 y no 0
                        if (bytesLeidos == 0 || bytesLeidos == 1)
                        {
                            // El socket se ha cerrado
                            break;
                        }

                        // Añadir los bytes leídos a la lista
                        for (int i = 0; i < bytesLeidos; i++)
                        {
                            receivedBytes.Add(bufferSalida[i]);
                        }
                    }
                    else
                    {
                        Log.Information("Tiempo de espera agotado. No se recibieron más datos.");
                        break;
                    }
                    
                }

                byte[] data = receivedBytes.ToArray();

                // Convertir el array de bytes a una cadena
                string result = Encoding.UTF8.GetString(data);
                Log.Information($"XML de respuesta recibido por parte del POSBC:{result}");
                Entorno<EntornoPOSBC>.Instancia.get().BufferSalida = data;

                // Cerrar el socket
                socket.Shutdown(SocketShutdown.Both);
                socket.Close();
            }
            catch (SocketException ex)
            {
                Log.Error($"Error de Socket: {ex.Message}");
                Log.Error($"Código de error de Socket: {ex.SocketErrorCode}");
                Log.Error(ex.StackTrace);
            }
            catch (Exception ex)
            {
                Log.Error("Error:", ex.Message);
            }
            //Esto como se manejara? 
            return new Respuestas();
        }

        public IComando CreaCopia()
        {
            return (InitializeRequestPosbcCmd)this.MemberwiseClone();
        }

        public void CargaDTO(DTOBase initializeRequestDTO)
        {
            Request = (InitializeRequestDTO)initializeRequestDTO;
        }

    }
}
